# Andrey version 1.0.0
# AndreyLexer.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of Andrey, a pedagogical implementation of Markov chains.
# Andrey is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Andrey is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Andrey.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Token
# 3) Lexer

# 1) Imported modules
# -------------------

import string
import sys
import re

from S2MLLexer import *
from AndreyType import *

# 2) Lexer
# --------

# 2.1) Creator
# ------------

class AndreyLexer(S2MLLexer):

  def __init__(self, engine):
    S2MLLexer.__init__(self, engine)

# 2.2) Management of actions
# --------------------------

  def RegisterActions(self):
    S2MLLexer.RegisterActions(self)
    self.RegisterAction("-", AndreyLexer.ActionMinus)

  def ActionMinus(self, c):
    c = self.GetChar()
    if c.isdigit():
      self.tokenString = "-" + c
      self.state = S2MLLexer.NUMBER_1
    elif c==">":
      self.tokenString = "->"
      self.NewToken(AndreyType.TOKEN_RIGHT_ARROW)
    else:
      self.tokenString = "-"
      self.PutCharBack(c)
      self.NewToken(S2MLLexer.TOKEN_UNKNOWN)

# 2.3) Management of keywords
# ---------------------------

  def RegisterTokens(self):
    S2MLLexer.RegisterTokens(self)
    self.RegisterToken("->", AndreyType.TOKEN_RIGHT_ARROW)

  def RegisterKeywords(self):
    S2MLLexer.RegisterKeywords(self)
    self.RegisterKeyword("state", AndreyType.KEYWORD_STATE)
    self.RegisterKeyword("event", AndreyType.KEYWORD_EVENT)
    self.RegisterKeyword("reward", AndreyType.KEYWORD_REWARD)
    self.RegisterKeyword("transition", AndreyType.KEYWORD_TRANSITION)
    self.RegisterKeyword("in", AndreyType.KEYWORD_IN)
